<?php
/*
Plugin Name: Post Restrictions
Plugin URI: http://soeren-weber.net/post/2006/01/01/121/
Description: Only show posts to other users if they have the corresponding capability.
Version: 1.2
Author: S&ouml;ren Weber
Author URI: http://soeren-weber.net

This work is derived from Filipe Fortes Post Levels plugin (http://fortes.com/)

Known Issues:
 - Private and protected posts don't show up in the next/previous post links
*/

define('POSTRESTRICTIONS_VERSION', '1.2');
define('POSTRESTRICTIONS_PLUGIN_ID', 'post_restrictions');
define('POSTRESTRICTIONS_META_KEY', 'post_restrictions');

define('POSTRESTRICTIONS_POSTLEVELS_POST_META_KEY', 'post_level');
define('POSTRESTRICTIONS_POSTLEVELS_POST_META_KEY_OPTION', 'postlevels_post_key');
define('POSTRESTRICTIONS_POSTLEVELS_USER_META_KEY_OPTION', 'postlevels_user_key');
define('POSTRESTRICTIONS_POSTLEVELS_ADMIN_ROLE', 'post_level_admin');

define('POSTRESTRICTIONS_VIEWLEVELS_POST_META_KEY', 'viewlevel');
define('POSTRESTRICTIONS_VIEWLEVELS_DEFAULT_OPTION', 'vl_default');

define('POSTRESTRICTIONS_PRIVATECATEGORIES_CATS_OPTION', 'wwpc_cats');
define('POSTRESTRICTIONS_PRIVATECATEGORIES_LEVEL_OPTION', 'wwpc_level');

// ----------------------------------------------------------------------------
// migration functions
// ----------------------------------------------------------------------------

function postrestrictions_migrate_post_levels()
{
	global $wpdb, $wp_roles;

	// get the Post Level options
	$postlevels_post_meta_keys[] = POSTRESTRICTIONS_POSTLEVELS_POST_META_KEY;
	$postlevels_post_meta_key = get_option(POSTRESTRICTIONS_POSTLEVELS_POST_META_KEY_OPTION);
	if ($postlevels_post_meta_key !== false)
		$postlevels_post_meta_keys[] = $postlevels_post_meta_key;
	$postlevels_post_meta_in = "'". implode("', '", $postlevels_post_meta_keys). "'";

	$postlevels_user_meta_keys[] = '';
	$postlevels_user_meta_key = get_option(POSTRESTRICTIONS_POSTLEVELS_USER_META_KEY_OPTION);
	if ($postlevels_user_meta_key !== false)
		$postlevels_user_meta_keys[] = $postlevels_user_meta_key;
	$postlevels_user_meta_in = "'". implode("', '", $postlevels_user_meta_keys). "'";

	// find out whether plugin is installed
	$postlevels_installed = false;
	$query = "
		SELECT COUNT(*)
		FROM `{$wpdb->postmeta}`
		WHERE `meta_key` IN ($postlevels_post_meta_in)
		AND `meta_value` IN (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
	";
	if ($wpdb->get_var($query) > 0 && !in_array(POSTRESTRICTIONS_META_KEY, $postlevels_post_meta_keys))
		$postlevels_installed = true;

	// find out whether user level migration has been done
	$postlevels_user_migration = false;
	$query = "
		SELECT COUNT(*)
		FROM `{$wpdb->usermeta}`
		WHERE `meta_key` IN ($postlevels_user_meta_in)
		AND `meta_value` IN (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
	";
	if ($wpdb->get_var($query) > 0)
		$postlevels_user_migration = true;

	// do migration if requested
	if (isset($_POST['migrate_post_levels']) && $postlevels_installed)
	{
		$postlevels_capability_prefix = "level_";

		// create new capabilities if post levels 1.0 has migrated user levels
		if ($postlevels_user_migration)
		{
			// select user levels to convert to capablities
			$postlevels_capability_prefix = "read_". $postlevels_capability_prefix;
			$query = "
				SELECT `user_id`, `meta_value`
				FROM `{$wpdb->usermeta}`
				WHERE `meta_key` IN ($postlevels_user_meta_in)
				AND `meta_value` IN (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
			";
			$wpdb->query($query);
			$s = $wpdb->get_results($query);
			if (!is_array($s))
				$s = array();

			// create capabilities for this user and administrator role
			foreach ($s as $i)
			{
				// the admin role is needed because some role manager otherwise
				// do not recognize capabilities only assigned to users
				add_role(POSTLEVELS_ADMIN_ROLE, __("Post Levels Administrator", POSTRESTRICTIONS_PLUGIN_ID));
				$role = get_role(POSTLEVELS_ADMIN_ROLE);
				$role->add_cap("$postlevels_capability_prefix{$i->meta_value}");
				$user = new WP_User($i->user_id);
				$user->add_cap("$postlevels_capability_prefix{$i->meta_value}");
			}

			// select post levels to convert to capablities
			$query = "
				SELECT DISTINCT `meta_value`
				FROM `{$wpdb->postmeta}`
				WHERE `meta_key` IN ($postlevels_post_meta_in)
				AND `meta_value` IN (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
			";
			$wpdb->query($query);
			$s = $wpdb->get_results($query);
			if (!is_array($s))
				$s = array();

			// create post capabilities for administrator role
			foreach ($s as $i)
			{
				add_role(POSTLEVELS_ADMIN_ROLE, __("Post Levels Administrator", POSTRESTRICTIONS_PLUGIN_ID));
				$role = get_role(POSTLEVELS_ADMIN_ROLE);
				$role->add_cap("$postlevels_capability_prefix{$i->meta_value}");
			}

			// delete obsolete post levels user metas
			$query = "
				DELETE
				FROM `{$wpdb->usermeta}`
				WHERE `meta_key` IN ($postlevels_user_meta_in)
			";
			$wpdb->query($query);

			$postlevels_user_migration = false;
		}

		// changing post meta key values to that of Post Restrictions
		$query = "
			UPDATE `{$wpdb->postmeta}`
			SET `meta_key` = '". POSTRESTRICTIONS_META_KEY. "',
				`meta_value` = CONCAT('{$postlevels_capability_prefix}', `meta_value`)
			WHERE `meta_key` IN ($postlevels_post_meta_in)
			AND `meta_value` IN (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
		";
		$wpdb->query($query);

		$postlevels_installed = false;
		echo '<div id="message" class="updated fade"><p><strong>' . __('Post Levels migrated', POSTRESTRICTIONS_PLUGIN_ID) . "</strong></p></div>\n";
	}

	// create migration html if not already migrated
	if (!$postlevels_installed)
		return "";

	return '<div class="wrap">
	<h2>Migration from Post Levels</h2>
	<p>'. __('Post Restrictions has found that your blog used a version of the <a href="http://fortes.com/projects/wordpress/postlevels">Post Levels</a> plugin. If both plugins are active the functionality of each may behave unexpected! You can easily migrate your post levels to post restrictions by pressing the button below.</p>
	<p>The migration will either convert the previous set post_level to equivalent &quot;level_X&quot; capabilities or in case you have used Post Levels &quot;Migrate user levels&quot; feature it will convert the used post levels to a set of newly created &quot;read_level_X&quot; capabilites (where X is the number of the legacy user level). In case of newly created capabilities these will be assigned in equivalent manner to the users as they were previously assigend by the Post Levels plugin. Additionally a new &quot;Post Levels Administrator&quot; role will be created were all of the newly created capabilities been assigned to. These admin role is not yet assigned to any user after migration. You are free to delete it if you do not want to use it. It is just meant to simplify role/capability management after migration.</p>
	<p>Summary: After migration your blog behaves the same way as with Post Levels but is configured differently. You also should consider to install a Role Manager. See the common plugin repositories for implementations.</p>
	<p>Migration was tested with Post Levels version 1.0 but should work with all previous versions.</p>
<p><strong>Note, that migration can not be undone by this plugin once you have pressed the button below.</strong></p>', POSTRESTRICTIONS_PLUGIN_ID). '
	<form action="" method="post" id="post_levels_migrate">
		<p class="submit">
			<input type="submit" name="migrate_post_levels" value="' .__('Migrate Post Levels &raquo;', POSTRESTRICTIONS_PLUGIN_ID). '" />
		</p>
	</form>
</div>
';
}

function postrestrictions_migrate_view_levels()
{
	global $wpdb;

	// get the plugin options
	$viewlevels_post_meta_keys[] = POSTRESTRICTIONS_VIEWLEVELS_POST_META_KEY;
	$viewlevels_post_meta_in = "'". implode("', '", $viewlevels_post_meta_keys). "'";
	$viewlevels_default = get_option(POSTRESTRICTIONS_VIEWLEVELS_DEFAULT_OPTION);
	if ($viewlevels_default === false)
		$viewlevels_default = -1;

	// find out whether plugin is installed
	$viewlevels_installed = false;
	$query = "
		SELECT COUNT(*)
		FROM `{$wpdb->postmeta}`
		WHERE `meta_key` IN ($viewlevels_post_meta_in)
		AND `meta_value` IN (-1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
	";
	if ($wpdb->get_var($query) > 0)
		$viewlevels_installed = true;

	// do migration if requested
	if (isset($_POST['migrate_view_levels']) && $viewlevels_installed)
	{
		// delete nonfunctional view levels settings
		$query = "
			SELECT `ID` as post_id
			FROM `{$wpdb->posts}`
			WHERE `post_status` NOT IN ('publish')
		";
		$s = $wpdb->get_results($query);
		if (!is_array($s))
			$s = array();

		$unchanged = array();
		foreach ($s as $i)
		{
			$unchanged[] = $i->post_id;
			$query = "
				DELETE
				FROM `{$wpdb->postmeta}`
				WHERE `meta_key` IN ($viewlevels_post_meta_in)
				AND `post_id` = {$i->post_id}
			";
			$wpdb->query($query);
		}
		$unchanged_in = "'". implode("', '", $unchanged). "'";

		// migrate explicitly set protected posts
		$query = "
			SELECT DISTINCT `post_id`
			FROM `{$wpdb->postmeta}`
			WHERE `meta_key` IN ($viewlevels_post_meta_in)
			AND `meta_value` IN (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
		";
		$s = $wpdb->get_results($query);
		if (!is_array($s))
			$s = array();

		$protected = array();
		foreach ($s as $i)
		{
			$protected[] = $i->post_id;
			$query = "
				UPDATE `{$wpdb->posts}`
				SET `post_status` = 'private'
				WHERE `post_status` IN ('publish')
				AND `ID` = {$i->post_id}
			";
			$wpdb->query($query);
		}
		$protected_in = "'". implode("', '", $protected). "'";

		$query = "
			UPDATE `{$wpdb->postmeta}`
			SET `meta_key` = '". POSTRESTRICTIONS_META_KEY. "',
				`meta_value` = CONCAT('level_', `meta_value`)
			WHERE `meta_key` IN ($viewlevels_post_meta_in)
			AND `meta_value` IN (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
		";
		$wpdb->query($query);


		// migrate explicitly set public posts
		$query = "
			SELECT DISTINCT `post_id`
			FROM `{$wpdb->postmeta}`
			WHERE `meta_key` IN ($viewlevels_post_meta_in)
			AND `meta_value` IN (-1)
		";
		$s = $wpdb->get_results($query);
		if (!is_array($s))
			$s = array();

		$publish = array();
		foreach ($s as $i)
		{
			// there was also a user level set to restrict access
			// to this post, so we do not make it publish here
			if (in_array($i->post_id, $protected))
				continue;
			$publish[] = $i->post_id;
		}
		$publish_in = "'". implode("', '", $publish). "'";

		$query = "
			DELETE
			FROM `{$wpdb->postmeta}`
			WHERE `meta_key` IN ($viewlevels_post_meta_in)
			AND `meta_value` = -1;
		";
		$wpdb->query($query);

		if ($viewlevels_default > -1)
		{
			// migrate default view level
			$query = "
				UPDATE `{$wpdb->posts}`
				SET `post_status` = 'private'
				WHERE `post_status` IN ('publish')
				AND `ID` NOT IN ($unchanged_in)
				AND `ID` NOT IN ($publish_in)
				AND `ID` NOT IN ($protected_in)
			";
			$wpdb->query($query);

			$query = "
				SELECT `ID` AS post_id
				FROM `{$wpdb->posts}`
				WHERE `ID` NOT IN ($unchanged_in)
				AND `ID` NOT IN ($publish_in)
				AND `ID` NOT IN ($protected_in)
			";
			$wpdb->query($query);
			$s = $wpdb->get_results($query);
			if (!is_array($s))
				$s = array();

			foreach ($s as $i)
			{
				$query = "
					INSERT INTO `{$wpdb->postmeta}`
					(`post_id`, `meta_key`, `meta_value`)
					VALUES ({$i->post_id}, '". POSTRESTRICTIONS_META_KEY. "', 'level_{$viewlevels_default}')
				";
				$wpdb->query($query);
			}
		}

		// delete obsolete view levels postmetas
		$query = "
			DELETE
			FROM `{$wpdb->postmeta}`
			WHERE `meta_key` IN ($viewlevels_post_meta_in)
		";
		$wpdb->query($query);

		$viewlevels_installed = false;
		echo '<div id="message" class="updated fade"><p><strong>' . __('View Levels migrated', POSTRESTRICTIONS_PLUGIN_ID) . "</strong></p></div>\n";
	}

	// create migration html if not already migrated
	if (!$viewlevels_installed)
		return "";

	return '<div class="wrap">
	<h2>Migration from View Levels</h2>
	<p>'. __('Post Restrictions has found that your blog used a version of the <a href="http://blog.firetree.net/2005/08/25/viewlevel-20/">View Levels</a> plugin. If both plugins are active the functionality of each may behave unexpected! You can easily migrate your view levels to post restrictions by pressing the button below.</p>
	<p>Summary: After migration your blog\'s already published posts are converted to be protected by Post Restrictions according to your previous View Levels configuration. Generally Post Restrictions is more restrictive to your protected posts. E.g. they will not be published at an update service. Remember that there is no default restriction applied to new posts anymore. You also should consider to install a Role Manager. See the common plugin repositories for implementations.</p>
	<p>Migration was tested with ViewLevel2 version 2.0.0 but should work with all previous versions including ViewLevel.</p>
	<p><strong>Note, that migration can not be undone by this plugin once you have pressed the button below.</strong></p>', POSTRESTRICTIONS_PLUGIN_ID). '
	<form action="" method="post" id="view_levels_migrate">
		<p class="submit">
			<input type="submit" name="migrate_view_levels" value="'. __('Migrate View Levels &raquo;', POSTRESTRICTIONS_PLUGIN_ID). '" />
		</p>
	</form>
</div>
';
}

function postrestrictions_migrate_private_categories()
{
	global $wpdb;

	// get the plugin options
	$privatecategories_level = get_option(POSTRESTRICTIONS_PRIVATECATEGORIES_LEVEL_OPTION);
	if ($privatecategories_level < 0)
		$privatecategories_level = 0;
	else if ($privatecategories_level > 10)
		$privatecategories_level = 10;
	$privatecategories_cats = get_option(POSTRESTRICTIONS_PRIVATECATEGORIES_CATS_OPTION);
	$privatecategories_installed = false;
	if (is_array($privatecategories_cats) && count($privatecategories_cats) > 0)
		$privatecategories_installed = true;

	// do migration if requested
	if (isset($_POST['migrate_private_categories']) && $privatecategories_installed)
	{
		$privatecategories_cats_in = "'". implode("', '", $privatecategories_cats). "'";
		$query = "
			SELECT DISTINCT post.`ID` as post_id
			FROM `{$wpdb->posts}` AS post, `{$wpdb->post2cat}` AS post2cat
			WHERE post.`ID` = post2cat.`post_id`
			AND post2cat.`category_id` NOT IN ($privatecategories_cats_in)
			AND `post_status` IN ('publish')
		";
		$s = $wpdb->get_results($query);
		if (!is_array($s))
			$s = array();

		$publish = array();
		foreach ($s as $i)
			$publish[] = $i->post_id;
		$publish_in = "'". implode("', '", $publish). "'";

		$query = "
			SELECT `ID` as post_id
			FROM `{$wpdb->posts}`
			WHERE `post_status` NOT IN ('publish')
		";
		$s = $wpdb->get_results($query);
		if (!is_array($s))
			$s = array();

		$unchanged = array();
		foreach ($s as $i)
			$unchanged[] = $i->post_id;
		$unchanged_in = "'". implode("', '", $unchanged). "'";

		$query = "
			SELECT `ID` AS post_id
			FROM `{$wpdb->posts}`
			WHERE `ID` NOT IN ($unchanged_in)
			AND `ID` NOT IN ($publish_in)
		";
		$wpdb->query($query);
		$s = $wpdb->get_results($query);
		if (!is_array($s))
			$s = array();

		foreach ($s as $i)
		{
			$query = "
				INSERT INTO `{$wpdb->postmeta}`
				(`post_id`, `meta_key`, `meta_value`)
				VALUES ({$i->post_id}, '". POSTRESTRICTIONS_META_KEY. "', 'level_{$privatecategories_level}')
			";
			$wpdb->query($query);
		}

		$query = "
			UPDATE `{$wpdb->posts}`
			SET `post_status` = 'private'
			WHERE `post_status` IN ('publish')
			AND `ID` NOT IN ($unchanged_in)
			AND `ID` NOT IN ($publish_in)
		";
		$wpdb->query($query);

		// cleaning up obsolete stuff
		delete_option(POSTRESTRICTIONS_PRIVATECATEGORIES_CATS_OPTION);
		delete_option(POSTRESTRICTIONS_PRIVATECATEGORIES_LEVEL_OPTION);

		$privatecategories_installed = false;
		echo '<div id="message" class="updated fade"><p><strong>' . __('Private categories migrated', POSTRESTRICTIONS_PLUGIN_ID) . "</strong></p></div>\n";
	}

	// create migration html if not already migrated
	if (!$privatecategories_installed)
		return "";

	return '<div class="wrap">
	<h2>Migration from Private Categories</h2>
	<p>'. __('Post Restrictions has found that your blog used a version of the <a href="http://gregelin.com/archives/?page_id=2">Private Categories</a> plugin. If both plugins are active the functionality of each may behave unexpected! You can easily migrate your private categories to post restrictions by pressing the button below.</p>
	<p>Summary: After migration your blog behaves the same way as with Private Categories but is configured differently. Now the author has to take care to restrict the post when he saves it instead of the post being automatically protected by choosing the right categories. You also should consider to install a Role Manager. See the common plugin repositories for implementations.</p>
	<p>Migration was tested with Private Categories 2 version 0.2.</p>
	<p><strong>Note, that migration can not be undone by this plugin once you have pressed the button below.</strong></p>', POSTRESTRICTIONS_PLUGIN_ID). '
	<form action="" method="post" id="private_categories_migrate">
		<p class="submit">
			<input type="submit" name="migrate_private_categories" value="'. __('Migrate Private Categories &raquo;', POSTRESTRICTIONS_PLUGIN_ID). '" />
		</p>
	</form>
</div>
';
}

function postrestrictions_enforce_constraints()
{
	// just do some clean up to be sure our metas are consistent;
	// after migration is is mandatory but we can do it everywhere
	global $wpdb;

	// selecting duplicates with equal value
	$query = "
		SELECT `post_id`, `meta_value`, count(*) AS `count`
		FROM `{$wpdb->postmeta}`
		WHERE `meta_key` = '". POSTRESTRICTIONS_META_KEY. "'
		GROUP BY `post_id`, `meta_value`
		HAVING `count` > 1
	";
	$s = $wpdb->get_results($query);
	if (!is_array($s))
		$s = array();

	// deleting duplicates with equal value
	foreach ($s as $i)
	{
		$query = "
			DELETE
			FROM `{$wpdb->postmeta}`
			WHERE `meta_key` = '". POSTRESTRICTIONS_META_KEY. "'
			AND `post_id` = {$i->post_id}
			AND `meta_value` = '{$i->meta_value}'
			LIMIT ". strval($i->count - 1). "
		";
		$wpdb->query($query);
	}
}

// ----------------------------------------------------------------------------
// admin filter
// ----------------------------------------------------------------------------

// Post Restrictions configuration page
function postrestrictions_config_page()
{
	global $wpdb;
	global $g_postrestrictions_options;
	$feed_options = array('full_content', 'excerpt_only', 'title_only');

	// executing form actions
	if (isset($_POST['submit']))
	{
		check_admin_referer();
		$g_postrestrictions_options = array(
			'feed_privacy' => (in_array($_POST['post_restrictions_feed_privacy'], $feed_options)) ?
				$_POST['post_restrictions_feed_privacy'] : 'title_only',
			'rewrite_feed_link' => isset($_POST['post_restrictions_rewrite_feed_link'])
		);
		update_option(POSTRESTRICTIONS_PLUGIN_ID, $g_postrestrictions_options);
		echo '<div id="message" class="updated fade"><p><strong>' . __('Options saved.', POSTRESTRICTIONS_PLUGIN_ID) . "</strong></p></div>\n";
	}

	$html = "";
	$html .= postrestrictions_migrate_post_levels();
	$html .= postrestrictions_migrate_view_levels();
	$html .= postrestrictions_migrate_private_categories();

	postrestrictions_enforce_constraints();
?>
<div class="wrap">
	<h2><?php _e('Post Restrictions Options', POSTRESTRICTIONS_PLUGIN_ID); ?></h2>
	<p><?php _e("Post Restrictions shows private posts to other users if at least one of the user's capabilities meets one of the post's capabilities.", POSTRESTRICTIONS_PLUGIN_ID); ?></p>
	<form action="" method="post" id="post_restrictions_configuration">
		<table width="100%" cellspacing="2" cellpadding="5" class="editform">
			<tr valign="top">
				<th width="33%" scope="row"><?php _e('Feed Privacy:', POSTRESTRICTIONS_PLUGIN_ID); ?></th>
				<td>
					<select name="post_restrictions_feed_privacy" id="post_restrictions_feed_privacy">
<?php
						foreach ($feed_options as $option)
						{
							echo "						<option value='$option'";
							if ($option == $g_postrestrictions_options['feed_privacy'])
								echo " selected='yes'";
							echo ">$option</option>\n";
						}
?>
					</select>
					<br />
					<?php _e('For users reading feeds via HTTP authentication, this decides how much content should be shown', POSTRESTRICTIONS_PLUGIN_ID); ?>

				</td>
			</tr>

			<tr valign="top">
				<th scope="row"><?php _e('Rewrite feed links:', POSTRESTRICTIONS_PLUGIN_ID); ?></th>
				<td>
					<label for="post_restrictions_rewrite_feed_link">
						<input type="checkbox" name="post_restrictions_rewrite_feed_link" id="post_restrictions_rewrite_feed_link" value="true" <?php if ($g_postrestrictions_options['rewrite_feed_link']) : ?>checked="checked" <?php endif; ?>/>
						<?php _e('For logged in users, automatically appends <code>http_auth=yes</code> to the query string of feed links to support HTTP Authentication', POSTRESTRICTIONS_PLUGIN_ID); ?>

					</label>
				</td>
			</tr>
		</table>

		<p class="submit">
			<input type="submit" name="submit" value="<?php _e('Update Options') ?> &raquo;" />
		</p>
	</form>
</div>
<?php
	echo $html;
}

// Called when a post is saved, updates the post restriction
function postrestrictions_post_save($post_ID)
{
	global $current_user, $wpdb;
	global $g_postrestrictions_cap_names;

	// we don't update but recreate the whole postmetas because of lazyness
	$old_meta = get_post_meta($post_ID, POSTRESTRICTIONS_META_KEY, false);
	$query = "
		DELETE FROM `{$wpdb->postmeta}`
		WHERE `post_id` = '{$post_ID}'
		AND `meta_key` = '". POSTRESTRICTIONS_META_KEY. "'
	";
	$wpdb->query($query);
/* until we can not avoid user to delete metas, we can not restrict this
	foreach ($old_meta as $capability)
	{
		if (!$current_user->has_cap($capability))
			add_post_meta($post_ID, POSTRESTRICTIONS_META_KEY, $capability);
	}
*/
	foreach ($g_postrestrictions_cap_names as $capability)
	{
		if ($_POST["post_restrictions_$capability"] /* && $current_user->has_cap($capability) */)
		{
			add_post_meta($post_ID, POSTRESTRICTIONS_META_KEY, $capability);
		}
	}
}

// Called before post status is updated on save. Marks
// post as private if a post restriction is set
function postrestrictions_status_save($post_status)
{
	global $current_user;
	global $g_postrestrictions_cap_names;

	if ($post_status == 'publish')
	{
/* until we can not avoid user to delete metas, we can not restrict this
		$old_meta = get_post_meta($post_ID, POSTRESTRICTIONS_META_KEY, false);
		foreach ($old_meta as $capability)
		{
			if (!$current_user->has_cap($capability))
				return 'private';
		}
*/
		foreach ($g_postrestrictions_cap_names as $capability)
		{
			if ($_POST["post_restrictions_$capability"] /* && $current_user->has_cap($capability) */)
				return 'private';
		}
	}
	return $post_status;
}

// Called during the edit form, outputs the Post Restrictions drop down
function postrestrictions_do_form()
{
	global $post, $current_user;
	global $g_postrestrictions_cap_names;

	echo '<div class="dbx-box-wrapper">'."\n";
	echo '<fieldset id="post_restrictions" class="dbx-box">'."\n";
	echo '<div class="dbx-handle-wrapper">'."\n";
	echo '<h3 class="dbx-handle">'. __('Post Restrictions', POSTRESTRICTIONS_PLUGIN_ID). '</h3>'."\n";
	echo '</div>'."\n";
	echo '<div class="dbx-content-wrapper">'."\n";
	echo '<div class="dbx-content">'."\n";
	echo '<h3>'. __('Mark one or more capability to restrict access to this post. The post will then automatically marked as private and will not be published to the public:', POSTRESTRICTIONS_PLUGIN_ID). '</h3>'."\n";
	$restrictions = get_post_meta($post->ID, POSTRESTRICTIONS_META_KEY, false);
	foreach ($g_postrestrictions_cap_names as $capability)
	{
		$capability_name = ucwords(str_replace('_', ' ', $capability));
		echo '<label style="float: left; width: 15em;">';
		echo '<input type="checkbox" name="post_restrictions_'. $capability. '" id="post_restrictions_'. $capability. '"';
/* until we can not avoid user to delete metas, we can not restrict this
		if (!$current_user->has_cap($capability))
			echo ' disabled="disabled" readonly="readonly"';
*/
		if (is_array($restrictions) && in_array($capability, $restrictions))
			echo ' checked="checked" /><strong> '. $capability_name. '</strong>';
		else
			echo ' /> '. $capability_name;
		echo '</label>'. "\n";
	}
	echo '<div style="clear: both;"></div>'."\n";
	echo "</div>\n";
	echo "</div>\n";
	echo "</fieldset>\n";
	echo "</div>\n";
}

// Adds a post restriction column to the manage post list
function postrestrictions_add_column($columns)
{
	$columns[POSTRESTRICTIONS_PLUGIN_ID] = __('Restriction', POSTRESTRICTIONS_PLUGIN_ID);
	return $columns;
}

// Outputs the post restriction for each post in the manage post list
function postrestrictions_do_column($column_name)
{
	global $post;

	if ($column_name != POSTRESTRICTIONS_PLUGIN_ID)
		return false;

	$meta = get_post_meta($post->ID, POSTRESTRICTIONS_META_KEY, false);
	if ($post->post_status != 'private')
		_e('Public', POSTRESTRICTIONS_PLUGIN_ID);
	else if (!empty($meta))
		_e('<em>Protected</em>', POSTRESTRICTIONS_PLUGIN_ID);
	else
		_e('<strong>Private</strong>', POSTRESTRICTIONS_PLUGIN_ID);
	return true;
}

// ----------------------------------------------------------------------------
// runtime filter
// ----------------------------------------------------------------------------

// Enable users with the right restriction to read a private post
// This is required because single post viewing has special logic
// $allcaps = Capabilities the user currently has
// $caps = Primitive capabilities being tested / requested
// $args = array with:
// $args[0] = original meta capability requested
// $args[1] = user being tested
// $args[2] = post id to view
// See code for assumptions
function postrestrictions_must_fullfil($allcaps, $caps, $args)
{
	// This handler is only set up to deal with the read_private_posts
	// capability. Ignore all other calls into here.
	if (!in_array('read_private_posts', $caps))
		return $allcaps;

	// As of WP 2.0, read_private_posts is only requested when viewing
	// a single post. When this happens, the args[] array has three values,
	// as shown above. If WP changes the args[], this plugin will break

	$capabilities = array();
	$restrictions = get_post_meta($args[2], POSTRESTRICTIONS_META_KEY, false);
	$intersect = array_intersect(array_keys($allcaps), $restrictions);

	$capabilities['read_private_posts'] = !empty($intersect);
	return $capabilities;
}

// Left join with the custom fields table so we can filter
// based on restriction in the where clause
function postrestrictions_posts_join($join)
{
	// We add a left join here on the postmeta table --
	// this could conceivably break if another plugin
	// manipulates the join
	global $wpdb;
	$s = " LEFT JOIN {$wpdb->postmeta} AS postrestrictions_meta ON {$wpdb->posts}.ID = postrestrictions_meta.post_id
		AND postrestrictions_meta.meta_key = '". POSTRESTRICTIONS_META_KEY. "' ". $join;
	return $s;
}

// Change where clause to accept published posts and private posts
// with appropriate capability
function postrestrictions_posts_where($where)
{
	// This could break if another plugin manipulates the where
	// clause, specifically the post_status part
	global $wpdb, $current_user;
	$user_capabilities = "'". implode("', '", array_keys($current_user->allcaps)). "'";
	return str_replace('post_status = "publish"', '(post_status = "publish" OR (post_status = "private" ' .
		"AND (postrestrictions_meta.meta_key = '". POSTRESTRICTIONS_META_KEY. "' " .
			"AND postrestrictions_meta.meta_value IN ($user_capabilities) )))", $where);
}

// Called when using HTTP auth -- changes the post content
function postrestrictions_the_content_rss($content)
{
  global $post;
  global $g_postrestrictions_options;

  if ($post->post_status != 'private')
    return $content;

  switch ($g_postrestrictions_options['feed_privacy'])
  {
    case 'excerpt_only':
      return apply_filters('the_excerpt_rss', get_the_excerpt(true));

    case 'title_only':
    default:
      $permalink = get_permalink($post->ID);
      return __("View the content of this <a href='$permalink'>protected post</a>", POSTRESTRICTIONS_PLUGIN_ID);
  }
}

// Called when using HTTP auth -- changes the post excerpt
function postrestrictions_the_excerpt_rss($excerpt)
{
  global $post;
  global $g_postrestrictions_options;

  if ($post->post_status != 'private')
    return $excerpt;

  switch ($g_postrestrictions_options['feed_privacy'])
  {
    case 'excerpt_only':
      return $excerpt;

    case 'title_only':
    default:
      $permalink = get_permalink($post->ID);
      return __("View the content of this <a href='$permalink'>protected post</a>", POSTRESTRICTIONS_PLUGIN_ID);
  }
}

// Rewrites RSS feed links to support http authentication
// if the user is logged in
function postrestrictions_feed_link($output)
{
  $delim = (strpos($output, '?') === false) ? '?' : '&';
  return $output . $delim . 'http_auth=yes';
}

// ----------------------------------------------------------------------------
// setup
// ----------------------------------------------------------------------------

// See if we're doing HTTP Auth (http_auth=yes is in the query string)
function postrestrictions_enable_http_auth()
{
	if ($_GET['http_auth'] == 'yes')
	{
		// Perform HTTP user authentication against the WP user system
		if (!isset($_SERVER['PHP_AUTH_USER']) || !isset($_SERVER['PHP_AUTH_PW']))
		{
			header('WWW-Authenticate: Basic realm="' . get_bloginfo('name') . '"');
			header('HTTP/1.0 401 Unauthorized');
			die(__('You must be logged in to view this site', POSTRESTRICTIONS_PLUGIN_ID));
		}
		else
		{
			// Override WP's get_currentuserinfo in order to do the login
			// via HTTP auth. This code is copied from WP2.0's get_currentuserinfo
			// I wish there were a more targeted way to override this
			function get_currentuserinfo()
			{
				global $user_login, $userdata, $user_level, $user_ID, $user_email, $user_url, $user_pass_md5, $user_identity, $current_user;

				// Use HTTP auth instead of cookies
				if(!wp_login($_SERVER['PHP_AUTH_USER'], $_SERVER['PHP_AUTH_PW']))
				{
					header('WWW-Authenticate: Basic realm="' . get_bloginfo('name') . '"');
					header('HTTP/1.0 401 Unauthorized');
					die(__('Incorrect credentials', POSTRESTRICTIONS_PLUGIN_ID));
				}

				$user_login  = $_SERVER['PHP_AUTH_USER'];
				$userdata    = get_userdatabylogin($user_login);
				$user_level  = $userdata->user_level;
				$user_ID     = $userdata->ID;
				$user_email  = $userdata->user_email;
				$user_url    = $userdata->user_url;
				$user_pass_md5 = md5($userdata->user_pass);
				$user_identity = $userdata->display_name;

				if (empty($current_user))
					$current_user = new WP_User($user_ID);
			}
		}
	}
}

// Creates the database-stored options for the plugins that customize
// the plugin's behavior
function postrestrictions_setup_options()
{
	global $wp_roles;
	global $g_postrestrictions_options, $g_postrestrictions_cap_names;

	$g_postrestrictions_options = get_option(POSTRESTRICTIONS_PLUGIN_ID);
	if (!count($g_postrestrictions_options))
	{
		$g_postrestrictions_options = array(
			'feed_privacy' => 'title_only',
			'rewrite_feed_link' => true
		);
		add_option(POSTRESTRICTIONS_PLUGIN_ID, $g_postrestrictions_options);
	}

	// Get role list
	if ($wp_roles)
	{
		$g_postrestrictions_cap_names = array();
		foreach ($wp_roles->role_objects as $key => $role)
		{
			foreach ($role->capabilities as $capability => $grant)
				$g_postrestrictions_cap_names[$capability] = $capability;
		}
	}
	sort($g_postrestrictions_cap_names);
}

// Sets up the postrestrictions admin menu
function postrestrictions_admin_setup()
{
	global $wp_version;

	if (function_exists('add_submenu_page'))
	{
		add_submenu_page('options-general.php', __('Post Restrictions', POSTRESTRICTIONS_PLUGIN_ID),
			__('Post Restrictions', POSTRESTRICTIONS_PLUGIN_ID),
			8, __FILE__, 'postrestrictions_config_page');
	}

	add_filter('manage_posts_columns', 'postrestrictions_add_column');
	add_filter('manage_posts_custom_column', 'postrestrictions_do_column');

	if ($wp_version == "2.0")
		add_filter('edit_form_advanced', 'postrestrictions_do_form');
	else
		add_filter('dbx_post_advanced', 'postrestrictions_do_form');

	add_filter('status_save_pre', 'postrestrictions_status_save');
	add_filter('save_post', 'postrestrictions_post_save');
}

// Initialize plugin variables and functions, called during the
// 'init' action by WordPress
function postrestrictions_setup()
{
	// Setup options stored in database and global variables
	postrestrictions_setup_options();

	// Defer admin menu setup to only run if we're in admin section
	add_action('admin_menu', 'postrestrictions_admin_setup');

	// Don't bother with anything more unless someone is logged in
	if (is_user_logged_in())
	{
		// Make sure private pages aren't cached publicly
		header('Cache-Control: private');
		header('Pragma: no-cache');

		// Setup filters
		add_filter('posts_join', 'postrestrictions_posts_join');
		add_filter('posts_where', 'postrestrictions_posts_where');
		add_filter('user_has_cap', 'postrestrictions_must_fullfil', 10, 3);
		if ($g_postrestrictions_options['rewrite_feed_link'] == 1)
			add_filter('feed_link', 'postrestrictions_feed_link');

		// Only filter RSS content if we're using HTTP auth
		if ($_GET['http_auth'] == 'yes'
			&& $g_postrestrictions_options['feed_privacy'] != 'full_content')
		{
			// Don't bother setting up filters if we're not gonna change the content
			add_filter('the_content_rss', 'postrestrictions_the_content_rss');
			// WP 2.0 bug? the_content_rss never gets called
			add_filter('the_content', 'postrestrictions_the_content_rss');
			add_filter('the_excerpt_rss', 'postrestrictions_the_excerpt_rss');
		}

		// In case WP ever defines this function for us, check
		// if it exists first
		if (!function_exists('is_private'))
		{
			// Whether the current post is private
			function is_private()
			{
				global $post;
				$meta = get_post_meta($post->ID, POSTRESTRICTIONS_META_KEY, false);
				return ($post->post_status == 'private' && empty($meta));
			}
		}
		// Whether the current post is protected, means is marked private but has
		// post restrictions defined
		function is_protected()
		{
			global $post;
			$meta = get_post_meta($post->ID, POSTRESTRICTIONS_META_KEY, false);
			return ($post->post_status == 'private' && !empty($meta));
		}
		// Array with restrictions for this post
		function get_post_restrictions()
		{
			global $post;
			$meta = get_post_meta($post->ID, POSTRESTRICTIONS_META_KEY, false);
			return $meta;
		}
		function get_post_restrictions_list($display = true)
		{
			global $post;
			$meta = get_post_meta($post->ID, POSTRESTRICTIONS_META_KEY, false);
			$list = ucwords(str_replace('_', ' ', implode(", ", $meta)));
			if (!$display)
				return $list;
			echo $list;
		}
	}
	else
	{
		// Not logged in, define fast functions that don't use the DB
		// in case they're used in the templates
		if (!function_exists('is_private'))
		{
			function is_private()
			{
				return false;
			}
		}
		function is_protected()
		{
			return false;
		}
		function get_post_restrictions()
		{
			return array();
		}
		function get_post_restrictions_list($display = true)
		{
			return '';
		}
	}
}

// ----------------------------------------------------------------------------
// main
// ----------------------------------------------------------------------------

global $g_postrestrictions_cap_names;
$g_postrestrictions_cap_names = array();
global $g_postrestrictions_options;
$g_postrestrictions_options = array();

// Just in case someone's loaded up the page standalone for whatever reason,
// make sure it doesn't crash in an ugly way
if (!function_exists('add_filter'))
  die(__("This page must be loaded as part of WordPress", POSTRESTRICTIONS_PLUGIN_ID));

global $wp_version;
if (substr($wp_version, 0, 2) == "1.")
  die(__("This plugin requires at least WordPress 2.0", POSTRESTRICTIONS_PLUGIN_ID));

postrestrictions_enable_http_auth();

add_filter('init', 'postrestrictions_setup');
?>